
#include <iolpc2148.h>
#include "nRF24L01.h"
#include "const.h"

#define CSN_TIME      2
#define CE_HIGH_TIME  10000

unsigned char RX_ADDRESS_P0[5]  = {5,6,7,8,9};
unsigned char RX_ADDRESS_P1[5]  = {0,1,2,3,4};
unsigned char TX_ADDRESS[5]     = {5,6,7,8,9};
unsigned char ADDRESS[5];

unsigned char status;

void Delay (unsigned long a) { while (--a!=0); }


// Chip Select -> high
void CSN_HIGH (void)  { IO0SET_bit.P0_7=1; }
// Chip Select -> low
void CSN_LOW (void)   { Delay(CSN_TIME); IO0CLR_bit.P0_7=1; }

// Chip enable High
void CE_HIGH(void)    { IO0SET_bit.P0_2=1;  Delay(CE_HIGH_TIME); }
// Chip enable low
void CE_LOW(void)     { IO0CLR_bit.P0_2=1;}


// Init SPI0 Interface
void SPI0Init (void) {

  //port0.4 -> SCK0
  //port0.5 -> MISO0
  //port0.6 -> MOSI0
  PINSEL0_bit.P0_4 = 1;
  PINSEL0_bit.P0_5 = 1;
  PINSEL0_bit.P0_6 = 1;
  IO0DIR_bit.P0_4=1;
  IO0DIR_bit.P0_5=0;
  IO0DIR_bit.P0_6=1;

  // Chip select
  PINSEL0_bit.P0_7 = 0;
  IO0DIR_bit.P0_7 = 1;

  // Chip enable
  PINSEL0_bit.P0_2 = 0;
  IO0DIR_bit.P0_2 = 1;

  // IRQ pin
  PINSEL0_bit.P0_3 = 0;
  IO0DIR_bit.P0_3 = 0;

  // Set chip select and chip enable
  CSN_HIGH();
  CE_LOW();


  // Spi setting
  S0SPCR_bit.CPHA   = 0;      //ClockPhase (active)
  S0SPCR_bit.CPOL   = 0;      //ClockPlarity
  S0SPCR_bit.MSTR   = 1;      //Master mode (1) or Slave mode (0)
  S0SPCR_bit.LSBF   = 0;      //MSB First
  S0SPCR_bit.SPIE   = 0;      //Disable interrupt

  //Set Baund Rate
  S0SPCCR = 10;               //60MHz/4 / 10

}

// Transmit byte via SPI0 chanel
unsigned char  SPI_SendByte(unsigned char data) {

  // Send the data
  S0SPDR = data;

  // Wait until the character can be sent
  while(!S0SPSR_bit.SPIF);

  // return received data
  return S0SPDR;
}

unsigned char SPI_Send_command_with_ADDR (unsigned char cmd, unsigned char addr, unsigned char data_byte)
{
  unsigned char temp,command = 0;
  command = (cmd << 5) | addr;
  CSN_LOW();
  if (cmd == R_REGISTER)
  {
    if (addr == RX_ADDR_P0 || addr == RX_ADDR_P1 || addr == TX_ADDR)
    {

      status=SPI_SendByte(command);
      for (int k=0;k!=5;k++)
      {
        ADDRESS[k]=SPI_SendByte(NOP);
      }
      CSN_HIGH();
      return status;
    }
    else
    {
      status=SPI_SendByte(command);
      temp=SPI_SendByte(NOP);
      CSN_HIGH();
      return temp;
    }
  }
  if (cmd == W_REGISTER)
  {
    if (addr == RX_ADDR_P0)
    {
      status=SPI_SendByte(command);
      for (int j=0;j!=5;j++)
        {
          temp=RX_ADDRESS_P0[j];
          SPI_SendByte(temp);
        }
      CSN_HIGH();
      return status;
    }

    if (addr == RX_ADDR_P1)
    {
      status=SPI_SendByte(command);
      for (int j=0;j!=5;j++)
        {
          temp=RX_ADDRESS_P1[j];
          SPI_SendByte(temp);
        }
      CSN_HIGH();
      return status;
    }

    if (addr == TX_ADDR)
    {
      status=SPI_SendByte(command);
      for (int j=0;j!=5;j++)
        {
          temp=TX_ADDRESS[j];
          SPI_SendByte(temp);
        }
      CSN_HIGH();
      return status;
    }


    else
    {
      temp=SPI_SendByte(command);
      SPI_SendByte(data_byte);
      CSN_HIGH();
      return temp;
    }
  }

  return 1;
}

unsigned char SPI_Send_command_without_ADDR (unsigned char cmd, unsigned char data_byte)
{
  unsigned char temp = 0;
  CSN_LOW();
  if (cmd == R_RX_PAYLOAD)
  {
    status=SPI_SendByte(cmd);
    temp=SPI_SendByte(NOP);
    CSN_HIGH();
    return temp;
  }
  if (cmd == W_TX_PAYLOAD)
  {
    status=SPI_SendByte(cmd);
    SPI_SendByte(data_byte);
    CSN_HIGH();
    return status;
  }
  status = SPI_SendByte(cmd);
  CSN_HIGH();
  return status;

}

// Setting for nRF chip
void nRFSetting(void) {

  // Discard transmision
  CE_LOW();

  //Write CONFIG register (addres - 0x00)
  //00001010 - CRC enable, power-up, RX
  status = SPI_Send_command_with_ADDR(W_REGISTER, CONFIG_REG_ADDR, 0x0B);
  // read
  // config = SPI_Send_command_with_ADDR(R_REGISTER, CONFIG_REG_ADDR, NOP);

  //Write RX_ADDR_P0 register -> Set receive address data Pipe0 -> address in RX_ADDRESS_P0 array
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_ADDR_P0, NOP);
  // read
  // status = SPI_Send_command_with_ADDR(R_REGISTER, RX_ADDR_P0, NOP);

  //Write RX_ADDR_P1 register -> Set receive address data Pipe1 -> address in RX_ADDRESS_P1 array
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_ADDR_P1, NOP);
  // read
  // status = SPI_Send_command_with_ADDR(R_REGISTER,RX_ADDR_P1, NOP);

  //Write TX_ADDR register -> Transmit address. Used for a PTX device only. Address in TX_ADDRESS array
  status = SPI_Send_command_with_ADDR(W_REGISTER, TX_ADDR, NOP);
  // read
  // status = SPI_Send_command_with_ADDR(R_REGISTER, TX_ADDR, NOP);

  //Write RX_PW_P0 register -> Set number of bytes in RX payload in data pipe0 -> 1 byte
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_PW_P0, 1);
  // read
  // dummy = SPI_Send_command_with_ADDR(R_REGISTER, RX_PW_P0, NOP);

  //Write RX_PW_P1 register -> Set number of bytes in RX payload in data pipe1 -> 1 byte
  status = SPI_Send_command_with_ADDR(W_REGISTER, RX_PW_P1, 1);
  // read
  // dummy = SPI_Send_command_with_ADDR(R_REGISTER, RX_PW_P1, NOP);

}

